@@ -50,6 +50,10 @@ Value is one of the following symbols:
 	   (const ignore :tag "ignore marks without asking"))
   :group 'mu4e-headers)
 
+(defcustom mu4e-mark-execute-pre-hook nil
+  "Hook run just *before* a mark is applied to a message. The hook function
+is called with two arguments, the mark being executed and the message itself.")
+
 (defvar mu4e-headers-show-target t
   "Whether to show targets (such as '-> delete', '-> /archive')
 when marking message. Normally, this is useful information for the
@@ -101,32 +105,31 @@ where
 (defmacro mu4e~mark-in-context (&rest body)
   "Evaluate BODY in the context of the headers buffer in case this
 is either a headers or view buffer."
-  `(cond     
+  `(cond
      ((eq major-mode 'mu4e-headers-mode) ,@body)
-     ((eq major-mode 'mu4e-view-mode)  
-       (when (buffer-live-p mu4e~view-headers-buffer)
+     ((eq major-mode 'mu4e-view-mode)
+       (when (buffer-live-p (mu4e-get-headers-buffer))
 	 (let* ((msg (mu4e-message-at-point))
 		 (docid (mu4e-message-field msg :docid)))
-	   (with-current-buffer mu4e~view-headers-buffer
+	   (with-current-buffer (mu4e-get-headers-buffer)
 	     (if (mu4e~headers-goto-docid docid)
 	       ,@body
-	       (mu4e-error "cannot find message in headers buffer.")))))) 
+	       (mu4e-error "cannot find message in headers buffer."))))))
      (t
        ;; even in other modes (e.g. mu4e-main-mode we try to find
        ;; the headers buffer
        (let ((hbuf (mu4e~mark-find-headers-buffer)))
 	 (if (buffer-live-p hbuf)
-	   (with-current-buffer hbuf
-	     ,@body)
-	   (progn (mu4e-message "%S" major-mode) ,@body))))))  
+	   (with-current-buffer hbuf ,@body)
+	   ,@body)))))
 
 (defvar mu4e-marks
     '((refile
 	:char ("r" . "▶")
 	:prompt "refile"
 	:dyn-target (lambda (target msg) (mu4e-get-refile-folder msg))
-	:action (lambda (docid msg target) (mu4e~proc-move docid
-						  (mu4e~mark-check-target target) "-N")))
+	:action (lambda (docid msg target)
+		  (mu4e~proc-move docid (mu4e~mark-check-target target) "-N")))
        (delete
 	 :char ("D" . "❌")
 	 :prompt "Delete"
@@ -136,13 +139,14 @@ is either a headers or view buffer."
 	 :char ("+" . "✚")
 	 :prompt "+flag"
 	 :show-target (lambda (target) "flag")
-	 :action (lambda (docid msg target) (mu4e~proc-move docid nil "+F-u-N")))
+	 :action (lambda (docid msg target)
+		   (mu4e~proc-move docid nil "+F-u-N")))
        (move
 	 :char ("m" . "▷")
 	 :prompt "move"
 	 :ask-target  mu4e~mark-get-move-target
-	 :action (lambda (docid msg target) (mu4e~proc-move docid
-					      (mu4e~mark-check-target target) "-N")))
+	 :action (lambda (docid msg target)
+		   (mu4e~proc-move docid (mu4e~mark-check-target target) "-N")))
        (read
 	 :char    ("!" . "◼")
 	 :prompt "!read"
@@ -185,7 +189,7 @@ is either a headers or view buffer."
 	 :char  ("*" . "✱")
 	 :prompt "*something"
 	 :action (mu4e-error "No action for deferred mark")))
-       
+
   "The list of all the possible marks.
 This is an alist mapping mark symbols to their properties.  The
 properties are:
@@ -239,7 +243,8 @@ The following marks are available, and the corresponding props:
 	  ;; get a cell with the mark char and the 'target' 'move' already has a
 	  ;; target (the target folder) the other ones get a pseudo "target", as
 	  ;; info for the user.
-	  (markdesc (cdr (or (assq mark mu4e-marks) (mu4e-error "Invalid mark %S" mark))))
+	  (markdesc (cdr (or (assq mark mu4e-marks)
+			   (mu4e-error "Invalid mark %S" mark))))
 	  (get-markkar
 	   (lambda (char)
 	     (if (listp char)
@@ -250,13 +255,14 @@ The following marks are available, and the corresponding props:
 	  (show-fct (plist-get markdesc :show-target))
 	  (shown-target (if show-fct
 			  (funcall show-fct target)
-                          (if target (format "%S" target)))))
+			  (if target (format "%S" target)))))
     (unless docid (mu4e-warn "No message on this line"))
-    (unless (eq major-mode 'mu4e-headers-mode) (mu4e-error "Not in headers-mode"))
+    (unless (eq major-mode 'mu4e-headers-mode)
+      (mu4e-error "Not in headers-mode"))
     (save-excursion
       (when (mu4e~headers-mark docid markkar)
-	;; update the hash -- remove everything current, and if add the new stuff,
-	;; unless we're unmarking
+	;; update the hash -- remove everything current, and if add the new
+	;; stuff, unless we're unmarking
 	(remhash docid mu4e~mark-map)
 	;; remove possible overlays
 	(remove-overlays (line-beginning-position) (line-end-position))
@@ -268,8 +274,8 @@ The following marks are available, and the corresponding props:
 	  (when (and shown-target mu4e-headers-show-target)
 	    (let* ((targetstr (propertize (concat "-> " shown-target " ")
 				'face 'mu4e-system-face))
-		    ;; mu4e~headers-goto-docid docid t \will take us just after the
-		    ;; docid cookie and then we skip the mu4e~mark-fringe
+		    ;; mu4e~headers-goto-docid docid t \will take us just after
+		    ;; the docid cookie and then we skip the mu4e~mark-fringe
 		    (start (+ (length mu4e~mark-fringe)
 			     (mu4e~headers-goto-docid docid t)))
 		    (overlay (make-overlay start (+ start (length targetstr)))))
@@ -386,29 +392,33 @@ work well.
 If NO-CONFIRMATION is non-nil, don't ask user for confirmation."
   (interactive)
   (mu4e~mark-in-context
-    (let ((marknum (hash-table-count mu4e~mark-map)))
-      (if (zerop marknum)
-	(message "Nothing is marked")
-	(mu4e-mark-resolve-deferred-marks)
-	(when (or no-confirmation
-		(y-or-n-p
+   (let ((marknum (hash-table-count mu4e~mark-map)))
+     (if (zerop marknum)
+	 (message "Nothing is marked")
+       (mu4e-mark-resolve-deferred-marks)
+       (when (or no-confirmation
+		 (y-or-n-p
 		  (format "Are you sure you want to execute %d mark%s?"
-		    marknum (if (> marknum 1) "s" ""))))
-	  (maphash
-	    (lambda (docid val)
-	      (let* ((mark (car val)) (target (cdr val))
-		      (markdescr (assq mark mu4e-marks))
-		      (msg (save-excursion
-			     (mu4e~headers-goto-docid docid)
-			     (mu4e-message-at-point))))
-		;; note: whenever you do something with the message,
-		;; it looses its N (new) flag
-                (if markdescr
-		  (funcall (plist-get (cdr markdescr) :action) docid msg target)
-		  (mu4e-error "Unrecognized mark %S" mark))))
-	    mu4e~mark-map))
-	(mu4e-mark-unmark-all)
-	(message nil)))))
+			  marknum (if (> marknum 1) "s" ""))))
+	 (maphash
+	  (lambda (docid val)
+	    (let* ((mark (car val)) (target (cdr val))
+		   (markdescr (assq mark mu4e-marks))
+		   (msg (save-excursion
+			  (mu4e~headers-goto-docid docid)
+			  (mu4e-message-at-point))))
+	      ;; note: whenever you do something with the message,
+	      ;; it looses its N (new) flag
+	      (if markdescr
+		  (progn
+		    (run-hook-with-args
+		    'mu4e-mark-execute-pre-hook mark msg)
+		    (funcall (plist-get (cdr markdescr) :action)
+		      docid msg target))
+		(mu4e-error "Unrecognized mark %S" mark))))
+	  mu4e~mark-map))
+       (mu4e-mark-unmark-all)
+       (message nil)))))
 
 (defun mu4e-mark-unmark-all ()
   "Unmark all marked messages."
@@ -432,8 +442,8 @@ If NO-CONFIRMATION is non-nil, don't ask user for confirmation."
 
 (defun mu4e-mark-marks-num ()
   "Return the number of marks in the current buffer."
-  (if mu4e~mark-map (hash-table-count mu4e~mark-map) 0))
-
+  (mu4e~mark-in-context
+    (if mu4e~mark-map (hash-table-count mu4e~mark-map) 0)))
 
 (defun mu4e-mark-handle-when-leaving ()
   "If there are any marks in the current buffer, handle those
@@ -441,13 +451,14 @@ according to the value of `mu4e-headers-leave-behavior'. This
 function is to be called before any further action (like searching,
 quitting the buffer) is taken; returning t means 'take the following
 action', return nil means 'don't do anything'."
-  (mu4e~mark-in-context 
+  (mu4e~mark-in-context
     (let ((marknum (mu4e-mark-marks-num))
 	   (what mu4e-headers-leave-behavior))
       (unless (zerop marknum) ;; nothing to do?
 	(when (eq what 'ask)
 	  (setq what (mu4e-read-option
-		       (format  "There are %d existing mark(s); should we: " marknum)
+		       (format  "There are %d existing mark(s); should we: "
+			 marknum)
 		       '( ("apply marks"   . apply)
 			  ("ignore marks?" . ignore)))))
 	;; we determined what to do... now do it
